

// EMS Class - usage examples
// Copyright 1993,1994 Scott Diorio
// All rights reserved

#include <iostream.h>
#include "emsclass.hpp"

#define ON 1

// Declare the EMS object globally with debugging ON and error reporting
// at level 1

EmsKernel EMS(ON,1);

void main(void)
{
	// determine the amount of system pages free
	int pages;
	if(( pages=EMS.SystemPagesFree() ) ==-1)
		return;

	// Alloc 63 - 16K pages to the EMSkernel
	EMS.Alloc(63);

	// display pages free info
	cout << "System EMS Pages Free: " << pages << endl;
	cout << "EMS Pages Free:  " << EMS.AppPagesFree() << endl;;

	// declare an EMSbuf of 100,000 and set to all '*'s
	EMSbuf A(100000);
	memset(A,'*',100000);


	// When working with an EMSbuf created with the new operator
	// remember to always dereference it!
	// Make sure "*B" is used and not just "B" or else you will be
	// calling the standard memset() function and not the memset used
	// for EMSbuf objects.  If you call the wrong memset() function
	// then the results will be undefined and you will most likely crash
	// your system. Hint...Use OS/2 to run you compiler in!

	EMSbuf *B;
	B=new EMSbuf(100000);
	memset(*B,0,100000);

	// Accessing individual characters within an EMSbuf is easy
	// use the brackets operators []
	for(long t=0;t<100000;t++)
		{
			A[t]='X';
		}

	// When accessing individual characters within an EMSbuf created
	// with the new operator, make sure to dereference it! For example...
	for(t=0;t<100000;t++)
		{
			(*B)[t]='X';
		}

	// The '+' operator may be used to gain access to any point within the
	// EMSbuf object.  The '+' operator used in conjunction with an EMSbuf
	// object returns to the calling function an offset into the EMSbuf.
	// In the below example the B is copied to A 10,000 bytes at a time.
	// This operation could be done with one memcpy call but it was done
	// this way to show the usage of the operator '+'

	for(t=0;t<90000;t+=10000)
		{
			memcpy( A+t,*B+t,10000);
		}

	// Character buffers may be used interchangably with EMSbuf objects.
	char *tmp;
	tmp=A;
	tmp=A+50000;

	// When setting a char * to an EMSbuf object make sure to use
	// the char* before any other manipulations on the EMSbuf.  Otherwise,
	// by the time the char * is used, the PageFrame of the EMSbuf may
	// be pointing to somewhere other than what you want your char *
	// pointing to. Look at the example below


	tmp=A+40000;   // The Correct EMS PageFrames are loaded to make the pointer
								 // 'tmp' point 40,000 characters into EMSbuf A

	memcpy(A+80000,*B,10000); // The object A now has a different EMS PageFrame
														// loaded

	memcpy( *B, tmp, 10000);  // 'tmp' is no longer pointing to where it was
														// originally intended to, the results are
														// undefined.


	// As already stated...interoperability between EMSbuf objects and char
	// bufffers is easy

	tmp=new char(20000);
	memcpy( A,tmp,20000);
	memcpy( *B+40000,tmp, 20000);


	delete(tmp);
	delete(B);
}
